************************************************************************
        function xA0_(m2)
************************************************************************
*       scalar 1-point function with complex mass
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
************************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 xA0_,xA0_old,A(0:0),Auv(0:0),m2,mm2,cm2,cmp2
        integer*4 rank

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	xA0_ = xA0_old(m2)
c	return

	mm2 = m2
	if (.not.(abs(mm2).gt.1d-15)) mm2 = cmp2(mm2)

	rank = 0
        call A_cll(A,Auv,mm2,rank)

	xA0_ = A(0)

        end

***********************************************************************
        function B0_(p2,m0,m1,ext)
************************************************************************
*       scalar 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 B0_,B0_old,Buv(0:0,0:0),B(0:0,0:0)
        complex*16 p10,m02,m12
        integer ext
        integer*4 rank

c	B0_ = B0_old(p2,m0,m1,ext)
c	return

	rank = 0
	p10 = dcmplx(p2)
	m02 = dcmplx(m0**2)
	m12 = dcmplx(m1**2)
        call B_cll(B,Buv,p10,m02,m12,rank)

	B0_ = B(0,0)

	end

***********************************************************************
        function xB0_(p2,m02,m12,ext)
************************************************************************
*       scalar 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 xB0_,xB0_old,Buv(0:0,0:0),B(0:0,0:0)
        complex*16 p10,m02,m12,mm02,mm12,cm2,cmp2
        integer ext
        integer*4 rank

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	xB0_ = xB0_old(p2,m02,m12,ext)
c	return

	rank = 0
        p10  = p2
	mm02 = m02
	mm12 = m12
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)

        call B_cll(B,Buv,p10,mm02,mm12,rank)

	xB0_ = B(0,0)

	end

***********************************************************************
        function DB0_(p2,m0,m1,ext)
************************************************************************
*       derivative of scalar 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 DB0_,DB0_old,DB0
        complex*16 p10,m02,m12
        integer ext
        integer*4 rank

c	DB0_ = DB0_old(p2,m0,m1,ext)
c	return

	rank = 0
	p10 = dcmplx(p2)
	m02 = dcmplx(m0**2)
	m12 = dcmplx(m1**2)
	call DB0_cll(DB0,p10,m02,m12)

	DB0_ = DB0

	end

***********************************************************************
        function xDB0_(p2,m02,m12,ext)
************************************************************************
*       derivative of scalar 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 xDB0_,xDB0_old,DB0
        complex*16 p10,m02,m12,mm02,mm12,cm2,cmp2
        integer ext
        integer*4 rank

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	xDB0_ = xDB0_old(p2,m02,m12,ext)
c	return

	rank = 0
        p10  = p2
	mm02 = m02
	mm12 = m12
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)
	call DB0_cll(DB0,p10,mm02,mm12)

	xDB0_ = DB0

	end

***********************************************************************
        function B1_(p2,m0,m1,ext)
************************************************************************
*       derivative of coefficient of vector 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 B1_,B1_old,Buv(0:0,0:1),B(0:0,0:1)
        complex*16 p10,m02,m12
        integer ext
        integer*4 rank

c	B1_ = B1_old(p2,m0,m1,ext)
c	return

	rank = 1
	p10 = dcmplx(p2)
	m02 = dcmplx(m0**2)
	m12 = dcmplx(m1**2)
        call B_cll(B,Buv,p10,m02,m12,rank)

	B1_ = B(0,1)

	end

***********************************************************************
        function xB1_(p2,m02,m12,ext)
************************************************************************
*       derivative of coefficient of vector 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 xB1_,xB1_old,Buv(0:0,0:1),B(0:0,0:1)
        complex*16 p10,m02,m12,mm02,mm12,cm2,cmp2
        integer ext
        integer*4 rank

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	xB1_ = xB1_old(p2,m02,m12,ext)
c	return

	rank = 1
        p10  = p2
	mm02 = m02
	mm12 = m12
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)
        call B_cll(B,Buv,p10,mm02,mm12,rank)

	xB1_ = B(0,1)

	end

***********************************************************************
        function DB1_(p2,m0,m1,ext)
************************************************************************
*       derivative of coefficient of vector 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 DB1_,DB1_old,DB1
        complex*16 p10,m02,m12
        integer ext
        integer*4 rank

c	DB1_ = DB1_old(p2,m0,m1,ext)
c	return

	rank = 0
	p10 = dcmplx(p2)
	m02 = dcmplx(m0**2)
	m12 = dcmplx(m1**2)
	call DB1_cll(DB1,p10,m02,m12)

	DB1_ = DB1

	end

***********************************************************************
        function xDB1_(p2,m02,m12,ext)
************************************************************************
*       derivative of coefficient of vector 2-point function
*-----------------------------------------------------------------------
*       31.5.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (a-z)
        complex*16 xDB1_,xDB1_old,DB1
        complex*16 p10,m02,m12,mm02,mm12,cm2,cmp2
        integer ext
        integer*4 rank

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	xDB1_ = xDB1_old(p2,m02,m12,ext)
c	return

	rank = 0
        p10  = p2
	mm02 = m02
	mm12 = m12
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)
	call DB1_cll(DB1,p10,mm02,mm12)

	xDB1_ = DB1

	end

***********************************************************************
        subroutine xB012345_(p2,m02,m12,B0,B1,B2,B3,B4,B5,irank,ext)
***********************************************************************
*       2-point tensor coefficients with complex masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)

        integer ext,irank
        integer*4 rank
        complex*16 Buv(0:irank/2,0:irank),B(0:irank/2,0:irank)
        complex*16 p10,m02,m12,mm02,mm12,cm2,cmp2
	complex*16 B0,B1(1),B2(0:2,0:2),B3(0:2,0:2,2)
        complex*16 B4(0:2,0:2,0:2,0:2),B5(0:2,0:2,0:2,0:2,2)

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c        call xB012345_old(p2,m02,m12,B0,B1,B2,B3,B4,B5,irank,ext)
c	return

        p10  = p2
	mm02 = m02
	mm12 = m12
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)

	rank = irank
        call B_cll(B,Buv,p10,mm02,mm12,rank)

        B0 = B(0,0)

        if (rank.lt.1) goto 100
        B1(1) = B(0,1)
        B2(0,0) = B(1,0)

        if (rank.lt.2) goto 100
        B2(1,1) = B(0,2)
        B3(0,0,1) = B(1,1)

        if (rank.lt.3) goto 100
        B3(1,1,1) = B(0,3)
        B4(0,0,0,0) = B(2,0)
        B4(0,0,1,1) = B(1,2)

        if (rank.lt.4) goto 100
        B4(1,1,1,1) = B(0,4)
        B5(0,0,0,0,1) = B(2,1)
        B5(0,0,1,1,1) = B(1,3)

        if (rank.lt.5) goto 100
        B5(1,1,1,1,1) = B(0,5)

        if (rank.lt.6) goto 100
        write(*,*) 'xB012345_: rank>5 not supported: ',rank
        stop

100     continue

	end

***********************************************************************
        function C0_(q01,q12,q20,m0,m1,m2,ext)
***********************************************************************
*       3-point tensor coefficients with real masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)
        integer*4 rank
        integer ext
        complex*16 p10,p21,p20,m02,m12,m22,cm2,cmp2
        complex*16 Cuv(0:0,0:0,0:0),C(0:0/2,0:0,0:0),C0_,C0_old

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	C0_ = C0_old(q01,q12,q20,m0,m1,m2,ext)
c	return

        p10  = q01
        p21  = q12
        p20  = q20
	m02 = m0**2
	m12 = m1**2
	m22 = m2**2
	if (.not.(abs(p10).gt.1d-15)) p10 = cmp2(p10)
	if (.not.(abs(p21).gt.1d-15)) p21 = cmp2(p21)
	if (.not.(abs(p20).gt.1d-15)) p20 = cmp2(p20)
	if (.not.(abs(m02).gt.1d-15)) m02 = cmp2(m02)
	if (.not.(abs(m12).gt.1d-15)) m12 = cmp2(m12)
	if (.not.(abs(m22).gt.1d-15)) m22 = cmp2(m22)

	rank = 0
        call C_cll(C,Cuv,p10,p21,p20,m02,m12,m22,rank)

	C0_ = C(0,0,0)

	end

***********************************************************************
        function xC0_(q01,q12,q20,m02,m12,m22,ext)
***********************************************************************
*       3-point tensor coefficients with complex masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)
        integer*4 rank
        integer ext
        complex*16 p10,p21,p20,m02,m12,m22,cm2,cmp2
        complex*16 mm02,mm12,mm22
        complex*16 Cuv(0:0,0:0,0:0),C(0:0/2,0:0,0:0),xC0_

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

        p10  = q01
        p21  = q12
        p20  = q20
	mm02 = m02
	mm12 = m12
	mm22 = m22
	if (.not.(abs(p10).gt.1d-15)) p10 = cmp2(p10)
	if (.not.(abs(p21).gt.1d-15)) p21 = cmp2(p21)
	if (.not.(abs(p20).gt.1d-15)) p20 = cmp2(p20)
	if (.not.(abs(mm02).gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)
	if (.not.(abs(mm22).gt.1d-15)) mm22 = cmp2(mm22)

	rank = 0
        call C_cll(C,Cuv,p10,p21,p20,mm02,mm12,mm22,rank)

	xC0_ = C(0,0,0)

	end

***********************************************************************
        subroutine xC01234_(p1,p2,p3,M12,M22,M32,
     &                    C0,C1,C2,C3,C4,B0,B1,B2,B3,irank,ext)
***********************************************************************
*       3-point tensor coefficients with complex masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)
        integer*4 rank
        integer ext,irank
        complex*16 p10,p21,p20,m12,m22,m32,mm12,mm22,mm32,cm2,cmp2
        complex*16 Cuv(0:irank/2,0:irank,0:irank)	
	complex*16 C(0:irank/2,0:irank,0:irank)
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        COMPLEX*16 C4(0:2,0:2,0:2,0:2)
        complex*16 B0,B1(1),B2(0:2,0:2),B3(0:2,0:2,2)

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c        call xC01234_old(p1,p2,p3,M12,M22,M32,
c     &                    C0,C1,C2,C3,C4,B0,B1,B2,B3,irank,ext)
c	return

        p10  = p1
        p21  = p2
        p20  = p3
	mm12 = m12
	mm22 = m22
	mm32 = m32
	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(p21) .gt.1d-15)) p21  = cmp2(p21)
	if (.not.(abs(p20) .gt.1d-15)) p20  = cmp2(p20)
	if (.not.(abs(mm12).gt.1d-15)) mm12 = cmp2(mm12)
	if (.not.(abs(mm22).gt.1d-15)) mm22 = cmp2(mm22)
	if (.not.(abs(mm32).gt.1d-15)) mm32 = cmp2(mm32)

	rank = irank
        call C_cll(C,Cuv,p10,p21,p20,mm12,mm22,mm32,rank)

        C0 = C(0,0,0)
        if (rank.lt.1) return
        C1(1) = C(0,1,0)
        C1(2) = C(0,0,1)
        C2(0,0) = C(1,0,0)
        if (rank.lt.2) return
        C2(1,1) = C(0,2,0)
        C2(1,2) = C(0,1,1)
        C2(2,2) = C(0,0,2)
        C3(0,0,1) = C(1,1,0)
        C3(0,0,2) = C(1,0,1)
        if (rank.lt.3) return
        C3(1,1,1) = C(0,3,0)
        C3(1,1,2) = C(0,2,1)
        C3(1,2,2) = C(0,1,2)
        C3(2,2,2) = C(0,0,3)
        C4(0,0,0,0) = C(2,0,0)
        C4(0,0,1,1) = C(1,2,0)
        C4(0,0,1,2) = C(1,1,1)
        C4(0,0,2,2) = C(1,0,2)
        if (rank.lt.4) return
        C4(1,1,1,1) = C(0,4,0)
        C4(1,1,1,2) = C(0,3,1)
        C4(1,1,2,2) = C(0,2,2)
        C4(1,2,2,2) = C(0,1,3)
        C4(2,2,2,2) = C(0,0,4)
        if (rank.lt.5) return
        write(*,*) 'xC01234_: rank>4 not supported: ',rank
        stop

        end

***********************************************************************
        subroutine  xD012345_(P1,P2,P3,P4,P23,P12,M12,M22,M32,M42,
     *                     D0,D1,D2,D3,D4,D5,C0,C1,C2,C3,C4,irank,ext)
***********************************************************************
*       4-point tensor coefficients with complex masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)

        integer ext,irank
        integer*4 rank
        complex*16 p10,p21,p32,p30,p20,p31,m12,m22,m32,m42
        complex*16 mm12,mm22,mm32,mm42,cm2,cmp2
        complex*16 D(0:irank/2,0:irank,0:irank,0:irank)
        complex*16 Duv(0:irank/2,0:irank,0:irank,0:irank)
        complex*16 D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
        complex*16 D4(0:3,0:3,0:3,0:3),D5(0:3,0:3,0:3,0:3,3)
        complex*16 C0,C1(2),C2(0:2,0:2),C3(0:2,0:2,2)
        complex*16 C4(0:2,0:2,0:2,0:2)

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c        call xD012345_old(P1,P2,P3,P4,P23,P12,M12,M22,M32,M42,
c     *                     D0,D1,D2,D3,D4,D5,C0,C1,C2,C3,C4,irank,ext)
c	return

        p10 = p1
        p21 = p2
        p32 = p3
        p30 = p4
        p20 = p12
        p31 = p23
	mm12 = m12
	mm22 = m22
	mm32 = m32
	mm42 = m42

	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(p21) .gt.1d-15)) p21  = cmp2(p21)
	if (.not.(abs(p32) .gt.1d-15)) p32  = cmp2(p32)
	if (.not.(abs(p30) .gt.1d-15)) p30  = cmp2(p30)
	if (.not.(abs(p20) .gt.1d-15)) p20  = cmp2(p20)
	if (.not.(abs(p31) .gt.1d-15)) p31  = cmp2(p31)
	if (.not.(abs(mm12) .gt.1d-15)) mm12 = cmp2(mm12)
	if (.not.(abs(mm22) .gt.1d-15)) mm22 = cmp2(mm22)
	if (.not.(abs(mm32) .gt.1d-15)) mm32 = cmp2(mm32)
	if (.not.(abs(mm42) .gt.1d-15)) mm42 = cmp2(mm42)

	rank = irank
        call D_cll(D,Duv,p10,p21,p32,p30,p20,p31,
     &	           mm12,mm22,mm32,mm42,rank)

        D0 = D(0,0,0,0)
        if (rank.lt.1) return
        D1(1) = D(0,1,0,0)
        D1(2) = D(0,0,1,0)
        D1(3) = D(0,0,0,1)
        if (rank.lt.2) return
        D2(0,0) = D(1,0,0,0)
        D2(1,1) = D(0,2,0,0)
        D2(1,2) = D(0,1,1,0)
        D2(1,3) = D(0,1,0,1)
        D2(2,2) = D(0,0,2,0)
        D2(2,3) = D(0,0,1,1)
        D2(3,3) = D(0,0,0,2)
        if (rank.lt.3) return
        D3(0,0,1) = D(1,1,0,0)
        D3(0,0,2) = D(1,0,1,0)
        D3(0,0,3) = D(1,0,0,1)
        D3(1,1,1) = D(0,3,0,0)
        D3(1,1,2) = D(0,2,1,0)
        D3(1,1,3) = D(0,2,0,1)
        D3(1,2,2) = D(0,1,2,0)
        D3(1,2,3) = D(0,1,1,1)
        D3(1,3,3) = D(0,1,0,2)
        D3(2,2,2) = D(0,0,3,0)
        D3(2,2,3) = D(0,0,2,1)
        D3(2,3,3) = D(0,0,1,2)
        D3(3,3,3) = D(0,0,0,3)
        if (rank.lt.4) return
        D4(0,0,0,0) = D(2,0,0,0)
        D4(0,0,1,1) = D(1,2,0,0)
        D4(0,0,1,2) = D(1,1,1,0)
        D4(0,0,1,3) = D(1,1,0,1)
        D4(0,0,2,2) = D(1,0,2,0)
        D4(0,0,2,3) = D(1,0,1,1)
        D4(0,0,3,3) = D(1,0,0,2)
        D4(1,1,1,1) = D(0,4,0,0)
        D4(1,1,1,2) = D(0,3,1,0)
        D4(1,1,1,3) = D(0,3,0,1)
        D4(1,1,2,2) = D(0,2,2,0)
        D4(1,1,2,3) = D(0,2,1,1)
        D4(1,1,3,3) = D(0,2,0,2)
        D4(1,2,2,2) = D(0,1,3,0)
        D4(1,2,2,3) = D(0,1,2,1)
        D4(1,2,3,3) = D(0,1,1,2)
        D4(1,3,3,3) = D(0,1,0,3)
        D4(2,2,2,2) = D(0,0,4,0)
        D4(2,2,2,3) = D(0,0,3,1)
        D4(2,2,3,3) = D(0,0,2,2)
        D4(2,3,3,3) = D(0,0,1,3)
        D4(3,3,3,3) = D(0,0,0,4)
        D5(0,0,0,0,1) = D(2,1,0,0)
        D5(0,0,0,0,2) = D(2,0,1,0)
        D5(0,0,0,0,3) = D(2,0,0,1)
        D5(0,0,1,1,1) = D(1,3,0,0)
        D5(0,0,1,1,2) = D(1,2,1,0)
        D5(0,0,1,1,3) = D(1,2,0,1)
        D5(0,0,1,2,2) = D(1,1,2,0)
        D5(0,0,1,2,3) = D(1,1,1,1)
        D5(0,0,1,3,3) = D(1,1,0,2)
        D5(0,0,2,2,2) = D(1,0,3,0)
        D5(0,0,2,2,3) = D(1,0,2,1)
        D5(0,0,2,3,3) = D(1,0,1,2)
        D5(0,0,3,3,3) = D(1,0,0,3)

        if (rank.lt.5) return
        D5(1,1,1,1,1) = D(0,5,0,0)
        D5(1,1,1,1,2) = D(0,4,1,0)
        D5(1,1,1,1,3) = D(0,4,0,1)
        D5(1,1,1,2,2) = D(0,3,2,0)
        D5(1,1,1,2,3) = D(0,3,1,1)
        D5(1,1,1,3,3) = D(0,3,0,2)
        D5(1,1,2,2,2) = D(0,2,3,0)
        D5(1,1,2,2,3) = D(0,2,2,1)
        D5(1,1,2,3,3) = D(0,2,1,2)
        D5(1,1,3,3,3) = D(0,2,0,3)
        D5(1,2,2,2,2) = D(0,1,4,0)
        D5(1,2,2,2,3) = D(0,1,3,1)
        D5(1,2,2,3,3) = D(0,1,2,2)
        D5(1,2,3,3,3) = D(0,1,1,3)
        D5(1,3,3,3,3) = D(0,1,0,4)
        D5(2,2,2,2,2) = D(0,0,5,0)
        D5(2,2,2,2,3) = D(0,0,4,1)
        D5(2,2,2,3,3) = D(0,0,3,2)
        D5(2,2,3,3,3) = D(0,0,2,3)
        D5(2,3,3,3,3) = D(0,0,1,4)
        D5(3,3,3,3,3) = D(0,0,0,5)

c remaining D6 coefficients not needed
        if (rank.lt.6) return
        write(*,*) 'xD012345_: rank>5 not supported: ',rank
        stop

	end

***********************************************************************
        subroutine xE01234_new(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,
     &    m02,m12,m22,m32,m42,E0n,E1n,E2n,E3n,E4n,D0,D1,D2,D3,irank,ext)
***********************************************************************
*       5-point tensor coefficients with complex masses
*-----------------------------------------------------------------------
*       5.6.2015 Stefan Dittmaier
***********************************************************************

        use COLLIER

        implicit real*8 (A-Z)
        integer ext,irank
        integer*4 rank
        complex*16 E0n,E1n(4),E2n(0:4,0:4),E3n(0:4,0:4,4)
        complex*16 E4n(0:4,0:4,0:4,0:4)
        complex*16 D0,D1(3),D2(0:3,0:3),D3(0:3,0:3,3)
        complex*16 p10,p21,p32,p43,p40,p20,p31,p42,p30,p41
	complex*16 m02,m12,m22,m32,m42,mm02,mm12,mm22,mm32,mm42,cm2,cmp2
        complex*16 E(0:irank/2,0:irank,0:irank,0:irank,0:irank)
        complex*16 Euv(0:irank/2,0:irank,0:irank,0:irank,0:irank)

        complex*16 cme2,cmf2(5)
        real*8 mf2(5),rmf2(5)
        common/fmass/me2,rme2,cme2,mf2,rmf2,cmf2

        cmp2(cm2) = mf2(nint(dreal(cm2*1d20)))

c	call xE01234_new_old(p1,p2,p3,p4,p5,s12,s23,s34,s45,s15,
c     &    m02,m12,m22,m32,m42,E0n,E1n,E2n,E3n,E4n,D0,D1,D2,D3,irank,ext)
c	return

        p10 = p1
        p21 = p2
        p32 = p3
        p43 = p4
        p40 = p5
        p20 = s12
        p31 = s23
        p42 = s34
        p30 = s45
        p41 = s15
	mm02 = m02
	mm12 = m12
	mm22 = m22
	mm32 = m32
	mm42 = m42

	if (.not.(abs(p10) .gt.1d-15)) p10  = cmp2(p10)
	if (.not.(abs(p21) .gt.1d-15)) p21  = cmp2(p21)
	if (.not.(abs(p32) .gt.1d-15)) p32  = cmp2(p32)
	if (.not.(abs(p43) .gt.1d-15)) p43  = cmp2(p43)
	if (.not.(abs(p40) .gt.1d-15)) p40  = cmp2(p40)
	if (.not.(abs(p20) .gt.1d-15)) p20  = cmp2(p20)
	if (.not.(abs(p31) .gt.1d-15)) p31  = cmp2(p31)
	if (.not.(abs(p42) .gt.1d-15)) p42  = cmp2(p42)
	if (.not.(abs(p30) .gt.1d-15)) p30  = cmp2(p30)
	if (.not.(abs(p41) .gt.1d-15)) p41  = cmp2(p41)
	if (.not.(abs(mm02) .gt.1d-15)) mm02 = cmp2(mm02)
	if (.not.(abs(mm12) .gt.1d-15)) mm12 = cmp2(mm12)
	if (.not.(abs(mm22) .gt.1d-15)) mm22 = cmp2(mm22)
	if (.not.(abs(mm32) .gt.1d-15)) mm32 = cmp2(mm32)
	if (.not.(abs(mm42) .gt.1d-15)) mm42 = cmp2(mm42)

	rank = irank
        call E_cll(E,Euv,p10,p21,p32,p43,p40,p20,p31,p42,p30,p41,
     &                       mm02,mm12,mm22,mm32,mm42,rank)

        E0n = E(0,0,0,0,0)
        if (rank.lt.1) return
        E1n(1) = E(0,1,0,0,0)
        E1n(2) = E(0,0,1,0,0)
        E1n(3) = E(0,0,0,1,0)
        E1n(4) = E(0,0,0,0,1)
        if (rank.lt.2) return
        E2n(0,0) = E(1,0,0,0,0)
        E2n(1,1) = E(0,2,0,0,0)
        E2n(1,2) = E(0,1,1,0,0)
        E2n(1,3) = E(0,1,0,1,0)
        E2n(1,4) = E(0,1,0,0,1)
        E2n(2,2) = E(0,0,2,0,0)
        E2n(2,3) = E(0,0,1,1,0)
        E2n(2,4) = E(0,0,1,0,1)
        E2n(3,3) = E(0,0,0,2,0)
        E2n(3,4) = E(0,0,0,1,1)
        E2n(4,4) = E(0,0,0,0,2)
        if (rank.lt.3) return
        E3n(0,0,1) = E(1,1,0,0,0)
        E3n(0,0,2) = E(1,0,1,0,0)
        E3n(0,0,3) = E(1,0,0,1,0)
        E3n(0,0,4) = E(1,0,0,0,1)
        E3n(1,1,1) = E(0,3,0,0,0)
        E3n(1,1,2) = E(0,2,1,0,0)
        E3n(1,1,3) = E(0,2,0,1,0)
        E3n(1,1,4) = E(0,2,0,0,1)
        E3n(1,2,2) = E(0,1,2,0,0)
        E3n(1,2,3) = E(0,1,1,1,0)
        E3n(1,2,4) = E(0,1,1,0,1)
        E3n(1,3,3) = E(0,1,0,2,0)
        E3n(1,3,4) = E(0,1,0,1,1)
        E3n(1,4,4) = E(0,1,0,0,2)
        E3n(2,2,2) = E(0,0,3,0,0)
        E3n(2,2,3) = E(0,0,2,1,0)
        E3n(2,2,4) = E(0,0,2,0,1)
        E3n(2,3,3) = E(0,0,1,2,0)
        E3n(2,3,4) = E(0,0,1,1,1)
        E3n(2,4,4) = E(0,0,1,0,2)
        E3n(3,3,3) = E(0,0,0,3,0)
        E3n(3,3,4) = E(0,0,0,2,1)
        E3n(3,4,4) = E(0,0,0,1,2)
        E3n(4,4,4) = E(0,0,0,0,3)
        if (rank.lt.4) return
        E4n(0,0,0,0) = E(2,0,0,0,0)
        E4n(0,0,1,1) = E(1,2,0,0,0)
        E4n(0,0,1,2) = E(1,1,1,0,0)
        E4n(0,0,1,3) = E(1,1,0,1,0)
        E4n(0,0,1,4) = E(1,1,0,0,1)
        E4n(0,0,2,2) = E(1,0,2,0,0)
        E4n(0,0,2,3) = E(1,0,1,1,0)
        E4n(0,0,2,4) = E(1,0,1,0,1)
        E4n(0,0,3,3) = E(1,0,0,2,0)
        E4n(0,0,3,4) = E(1,0,0,1,1)
        E4n(0,0,4,4) = E(1,0,0,0,2)
        E4n(1,1,1,1) = E(0,4,0,0,0)
        E4n(1,1,1,2) = E(0,3,1,0,0)
        E4n(1,1,1,3) = E(0,3,0,1,0)
        E4n(1,1,1,4) = E(0,3,0,0,1)
        E4n(1,1,2,2) = E(0,2,2,0,0)
        E4n(1,1,2,3) = E(0,2,1,1,0)
        E4n(1,1,2,4) = E(0,2,1,0,1)
        E4n(1,1,3,3) = E(0,2,0,2,0)
        E4n(1,1,3,4) = E(0,2,0,1,1)
        E4n(1,1,4,4) = E(0,2,0,0,2)
        E4n(1,2,2,2) = E(0,1,3,0,0)
        E4n(1,2,2,3) = E(0,1,2,1,0)
        E4n(1,2,2,4) = E(0,1,2,0,1)
        E4n(1,2,3,3) = E(0,1,1,2,0)
        E4n(1,2,3,4) = E(0,1,1,1,1)
        E4n(1,2,4,4) = E(0,1,1,0,2)
        E4n(1,3,3,3) = E(0,1,0,3,0)
        E4n(1,3,3,4) = E(0,1,0,2,1)
        E4n(1,3,4,4) = E(0,1,0,1,2)
        E4n(1,4,4,4) = E(0,1,0,0,3)
        E4n(2,2,2,2) = E(0,0,4,0,0)
        E4n(2,2,2,3) = E(0,0,3,1,0)
        E4n(2,2,2,4) = E(0,0,3,0,1)
        E4n(2,2,3,3) = E(0,0,2,2,0)
        E4n(2,2,3,4) = E(0,0,2,1,1)
        E4n(2,2,4,4) = E(0,0,2,0,2)
        E4n(2,3,3,3) = E(0,0,1,3,0)
        E4n(2,3,3,4) = E(0,0,1,2,1)
        E4n(2,3,4,4) = E(0,0,1,1,2)
        E4n(2,4,4,4) = E(0,0,1,0,3)
        E4n(3,3,3,3) = E(0,0,0,4,0)
        E4n(3,3,3,4) = E(0,0,0,3,1)
        E4n(3,3,4,4) = E(0,0,0,2,2)
        E4n(3,4,4,4) = E(0,0,0,1,3)
        E4n(4,4,4,4) = E(0,0,0,0,4)
        if (rank.lt.5) return
        write(*,*) 'xE01234_new: rank>4 not supported: ',rank
        stop

	end
